@@ -55,6 +55,8 @@ struct CmdOrPage {
 #define CMDFLAG_HIDDEN       0x0800     /* Elide from most listings */
 #define CMDFLAG_LDAVG_EXEMPT 0x1000     /* Exempt from load_control() */
 #define CMDFLAG_ALIAS        0x2000     /* Command aliases */
+#define CMDFLAG_KEEPEMPTY    0x4000     /* Do not unset empty settings */
+#define CMDFLAG_ABBREVSUBCMD 0x8000     /* Help text abbreviates subcommands */
 /**************************************************************************/
 
 /* Values for the 2nd parameter to dispatch_name_search() */
@@ -449,18 +451,27 @@ static void help_to_html(const char *zHelp, Blob *pHtml){
         iLevel++;
         aIndent[iLevel] = nIndent;
         azEnd[iLevel] = zEndUL;
+        if( wantP ){
+          blob_append(pHtml,"<p>", 3);
+          wantP = 0;
+        }
         blob_append(pHtml, "<ul>\n", 5);
-      }else if( isDT 
+      }else if( isDT
              || zHelp[nIndent]=='-'
              || hasGap(zHelp+nIndent,i-nIndent) ){
         iLevel++;
         aIndent[iLevel] = nIndent;
         azEnd[iLevel] = zEndDL;
+        wantP = 0;
         blob_append(pHtml, "<blockquote><dl>\n", -1);
       }else if( azEnd[iLevel]==zEndDL ){
         iLevel++;
         aIndent[iLevel] = nIndent;
         azEnd[iLevel] = zEndDD;
+        if( wantP ){
+          blob_append(pHtml,"<p>", 3);
+          wantP = 0;
+        }
         blob_append(pHtml, "<dd>", 4);
       }else if( wantP ){
         iLevel++;
@@ -514,9 +525,17 @@ static void help_to_html(const char *zHelp, Blob *pHtml){
 /*
 ** Format help text for TTY display.
 */
-static void help_to_text(const char *zHelp, Blob *pText){
+static void help_to_text(const char *zHelp, Blob *pText, int bUsage){
   int i, x;
   char c;
+  if( zHelp[0]=='>' ){
+    if( !bUsage ){
+      blob_appendf(pText, "Usage:");
+    }else{
+      blob_append_char(pText, ' ');
+    }
+    zHelp++;
+  }
   for(i=0; (c = zHelp[i])!=0; i++){
     if( c=='%' && strncmp(zHelp+i,"%fossil",7)==0 ){
       if( i>0 ) blob_append(pText, zHelp, i);
@@ -539,11 +558,11 @@ static void help_to_text(const char *zHelp, Blob *pText){
       zHelp += x-1;
       i = -1;
       continue;
-    }     
+    }
   }
   if( i>0 ){
     blob_append(pText, zHelp, i);
-  }      
+  }
 }
 
 /*
@@ -595,7 +614,7 @@ static void display_all_help(int mask, int useHtml, int rawOut){
       }else{
           Blob txt;
           blob_init(&txt, 0, 0);
-          help_to_text(aCommand[i].zHelp, &txt);
+          help_to_text(aCommand[i].zHelp, &txt, 0);
           for(j=0; j<occHelp[aCommand[i].iHelp]; j++){
             fossil_print("# %s%s\n",
               aCommand[bktHelp[aCommand[i].iHelp][j]].zName,
@@ -627,7 +646,7 @@ static void display_all_help(int mask, int useHtml, int rawOut){
 **
 ** Options:
 **    -a|--aliases      Show aliases
-**    -e|--everything   Show all commands and pages.  Omit aliases to 
+**    -e|--everything   Show all commands and pages.  Omit aliases to
 **                      avoid duplicates.
 **    -h|--html         Transform output to HTML
 **    -o|--options      Show global options
@@ -653,7 +672,7 @@ void test_all_help_cmd(void){
   }
   if( find_option("aliases","a",0) ){
     mask = CMDFLAG_ALIAS;
-  }  
+  }
   if( find_option("test","t",0) ){
     mask |= CMDFLAG_TEST;
   }
@@ -760,7 +779,7 @@ int dispatch_approx_match(const char *zIn, int nArray, const char **azArray){
   }
 
   while( n<nArray ){
-    bestScore = mxScore;    
+    bestScore = mxScore;
     for(i=iFirst; i<=iLast; i++){
       m = edit_distance(zIn, aCommand[i].zName);
       if( m<mnScore ) continue;
@@ -794,6 +813,28 @@ void test_approx_match_command(void){
   }
 }
 
+
+/*
+** Returns 1 if the command or page name zName is known to be a
+** command/page which is only available in certain builds/platforms,
+** else returns 0.
+*/
+static int help_is_platform_command(const char *zName){
+  const char *aList[] = {
+    /* List of commands/pages which are known to only be available in
+    ** certain builds/platforms. */
+    "winsrv",
+    "json", "/json",
+    NULL /* end-of-list sentinel */
+  };
+  int i = 0;
+  const char *z;
+  for( z = aList[0]; z ; z = aList[++i] ){
+    if( 0==fossil_strcmp(zName, z) ) return 1;
+  }
+  return 0;
+}
+
 /*
 ** WEBPAGE: help
 ** URL: /help?name=CMD
@@ -815,35 +856,48 @@ void help_page(void){
   const char *zCmd = P("cmd");
 
   if( zCmd==0 ) zCmd = P("name");
+  cgi_check_for_malice();
   if( zCmd && *zCmd ){
     int rc;
     const CmdOrPage *pCmd = 0;
 
-  style_set_current_feature("tkt");
-    style_header("Help: %s", zCmd);
-
-    style_submenu_element("Command-List", "%R/help");
+    style_set_current_feature("tkt");
+    style_submenu_element("Topic-List", "%R/help");
+    if( search_restrict(SRCH_HELP)!=0 ){
+      style_submenu_element("Search","%R/search?y=h");
+    }
     rc = dispatch_name_search(zCmd, CMDFLAG_ANY|CMDFLAG_PREFIX, &pCmd);
-    if( *zCmd=='/' ){
+    if( pCmd ){
+      style_header("Help: %s", pCmd->zName);
+    }else{
+      style_header("Help");
+    }
+    if( pCmd==0 ){
+      /* No <h1> line in this case */
+    }else if( *zCmd=='/' ){
       /* Some of the webpages require query parameters in order to work.
       ** @ <h1>The "<a href='%R%s(zCmd)'>%s(zCmd)</a>" page:</h1> */
-      @ <h1>The "%h(zCmd)" page:</h1>
+      @ <h1>The "%h(pCmd->zName)" page:</h1>
     }else if( rc==0 && (pCmd->eCmdFlags & CMDFLAG_SETTING)!=0 ){
       @ <h1>The "%h(pCmd->zName)" setting:</h1>
     }else{
-      @ <h1>The "%h(zCmd)" command:</h1>
+      @ <h1>The "%h(pCmd->zName)" command:</h1>
     }
-    if( rc==1 ){
-      @ unknown command: %h(zCmd)
+    if( rc==1 || (rc==2 && zCmd[0]=='/') ){
+      if( zCmd && help_is_platform_command(zCmd) ){
+        @ Not available in this build: "%h(zCmd)"
+      }else{
+        @ Unknown topic: "%h(zCmd)"
+      }
     }else if( rc==2 ){
-      @ ambiguous command prefix: %h(zCmd)
+      @ Ambiguous prefix: "%h(zCmd)"
     }else{
       if( pCmd->zHelp[0]==0 ){
         @ No help available for "%h(pCmd->zName)"
       }else if( P("plaintext") ){
         Blob txt;
         blob_init(&txt, 0, 0);
-        help_to_text(pCmd->zHelp, &txt);
+        help_to_text(pCmd->zHelp, &txt, 0);
         @ <pre class="helpPage">
         @ %h(blob_str(&txt))
         @ </pre>
@@ -860,13 +914,15 @@ void help_page(void){
     }
   }else{
     int i;
+    const char *zWidth = "28ex";
     unsigned char occHelp[FOSSIL_MX_CMDIDX] = {0};   /* Help str occurrences */
     int bktHelp[FOSSIL_MX_CMDIDX][MX_HELP_DUP] = {{0}};/* Help str->commands */
     style_header("Help");
+    search_screen(SRCH_HELP, 0x02);
 
     @ <a name='commands'></a>
     @ <h1>Available commands:</h1>
-    @ <div class="columns" style="column-width: 12ex;">
+    @ <div class="columns" style="column-width: %s(zWidth);">
     @ <ul>
     /* Fill in help string buckets */
     for(i=0; i<MX_COMMAND; i++){
@@ -888,7 +944,8 @@ void help_page(void){
         int aliases[MX_HELP_DUP], nAliases=0;
         for(j=0; j<occHelp[aCommand[i].iHelp]; j++){
           if( bktHelp[aCommand[i].iHelp][j] != i ){
-            if( aCommand[bktHelp[aCommand[i].iHelp][j]].eCmdFlags & CMDFLAG_ALIAS ){
+            if( aCommand[bktHelp[aCommand[i].iHelp][j]].eCmdFlags
+                & CMDFLAG_ALIAS ){
               aliases[nAliases++] = bktHelp[aCommand[i].iHelp][j];
             }
           }
@@ -910,7 +967,7 @@ void help_page(void){
 
     @ <a name='webpages'></a>
     @ <h1>Available web UI pages:</h1>
-    @ <div class="columns" style="column-width: 18ex;">
+    @ <div class="columns" style="column-width: %s(zWidth);">
     @ <ul>
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
@@ -926,7 +983,7 @@ void help_page(void){
 
     @ <a name='unsupported'></a>
     @ <h1>Unsupported commands:</h1>
-    @ <div class="columns" style="column-width: 20ex;">
+    @ <div class="columns" style="column-width: %s(zWidth);">
     @ <ul>
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
@@ -942,7 +999,7 @@ void help_page(void){
 
     @ <a name='settings'></a>
     @ <h1>Settings:</h1>
-    @ <div class="columns" style="column-width: 20ex;">
+    @ <div class="columns" style="column-width: %s(zWidth);">
     @ <ul>
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
@@ -978,7 +1035,7 @@ void test_all_help_page(void){
   for(i=0; i<MX_COMMAND; i++){
     if(aCommand[i].eCmdFlags & CMDFLAG_HIDDEN) continue;
     bktHelp[aCommand[i].iHelp][occHelp[aCommand[i].iHelp]++] = i;
-  }  
+  }
   for(i=0; i<MX_COMMAND; i++){
     const char *zDesc;
     unsigned int e = aCommand[i].eCmdFlags;
@@ -1030,13 +1087,13 @@ void test_all_help_page(void){
             zDesc = "web page";
           }
         }
-        
+
         @ <dt><big><b>%s(aCommand[bktHelp[aCommand[i].iHelp][j]].zName)</b>
         @</big> (%s(zDesc))</dt>
       }
-      @ <dd>
+      @ <p><dd>
       help_to_html(aCommand[i].zHelp, cgi_output_blob());
-      @ </dd>
+      @ </dd><p>
       occHelp[aCommand[i].iHelp] = 0;
     }
   }
@@ -1045,6 +1102,225 @@ void test_all_help_page(void){
   style_finish_page();
 }
 
+/*
+** Analyze p and return one of three values:
+**
+**     0     p is the continuation of a prior subcommand.
+**
+**     1     p is text past the end of a prior subcommand.
+**
+**     2     p is the start of a new subcommand.
+*/
+static int is_subcommand(Blob *p, int bAbbrevSubcmd){
+  int i, sz;
+  const unsigned char *z = (const unsigned char*)blob_buffer(p);
+  sz = blob_size(p);
+  if( sz>6 ) sz = 6;
+  for(i=0; i<sz && fossil_isspace(z[i]); i++){}
+  if( i>=sz ) return 0;
+
+  if( bAbbrevSubcmd==0 ){
+    if( i>1 ) return 0;
+    return z[0]=='>' ? 2 : 1;
+  }else{
+    return (i==3 && fossil_isalpha(z[3])) ? 2 : 1;
+  }
+}
+
+/*
+** Input z[] is help text for zTopic.  If zTopic has sub-command zSub,
+** then cut out all portions of the original help text that do not
+** directly pertain to zSub and write the zSub-relevant parts into
+** pOut.
+**
+** Return the number of lines of z[] written into pOut.  A return of
+** zero means no simplification occurred.
+*/
+static int simplify_to_subtopic(
+  const char *z,            /* Full original help text */
+  Blob *pOut,               /* Write simplified help text here */
+  const char *zTopic,       /* TOPIC */
+  const char *zSubtopic,    /* SUBTOPIC */
+  int bAbbrevSubcmd         /* True if z[] contains abbreviated subcommands */
+){
+  Blob in, line; //, subsection;
+  int n = 0;
+  char *zQTop = re_quote(zTopic);
+  char *zQSub = re_quote(zSubtopic);
+  char *zPattern;
+  ReCompiled *pRe = 0;
+
+  if( bAbbrevSubcmd ){
+    zPattern = mprintf("   ([a-z]+ ?\\| ?)*%s\\b", zQSub);
+  }else{
+    zPattern = mprintf(">  ?fossil [-a-z]+ .*\\b%s\\b", zQSub);
+  }
+  fossil_free(zQTop);
+  fossil_free(zQSub);
+  fossil_re_compile(&pRe, zPattern, 0);
+  fossil_free(zPattern);
+  blob_init(&in, z, -1);
+  while( blob_line(&in, &line) ){
+    if( re_match(pRe, (unsigned char*)blob_buffer(&line), blob_size(&line)) ){
+      int atStart = 1;
+      blob_appendb(pOut, &line);
+      n++;
+      while( blob_line(&in, &line) ){
+        if( re_match(pRe,(unsigned char*)blob_buffer(&line),blob_size(&line)) ){
+          blob_appendb(pOut, &line);
+          n++;
+          atStart = 1;
+        }else{
+          int x = is_subcommand(&line,bAbbrevSubcmd);
+          if( x==2 ){
+            if( atStart ){
+              blob_appendb(pOut, &line);
+              n++;
+            }else{
+              break;
+            }
+          }else if( x==1 ){
+            break;
+          }else{
+            blob_appendb(pOut, &line);
+            n++;
+            atStart = 0;
+          }
+        }
+      }
+    }
+  }
+  blob_reset(&line);
+  re_free(pRe);
+  if( n ){
+    blob_trim(pOut);
+    blob_reset(&in);
+  }
+  return n;
+}
+
+/*
+** Input p is a "Usage:" line or a subcommand line.  Simplify this line
+** for the --usage option and write it into pOut.
+*/
+static void simplify_usage_line(
+  Blob *p,
+  Blob *pOut,
+  int bAbbrevSubcmd,
+  const char *zCmd
+){
+  const char *z = blob_buffer(p);
+  int sz = blob_size(p);
+  int i = 0;
+  if( sz>6 && z[0]=='U' ){
+    for(i=1; i<sz && !fossil_isspace(z[i]); i++){}
+  }else if( sz>0 && z[0]=='>' ){
+    i = 1;
+  }else if( sz>4 && bAbbrevSubcmd
+         && memcmp(z,"   ",3)==0 && !fossil_isspace(z[3]) ){
+    int j;
+    for(j=3; j<sz-1 && (z[j]!=' ' || z[j+1]!=' '); j++){}
+    blob_appendf(pOut, "fossil %s %.*s\n", zCmd, j-3, &z[3]);
+    return;
+  }else{
+    while( i<sz && fossil_isspace(z[i]) ) i++;
+    if( i+2<sz && (z[i]=='o' || z[i]=='O') && z[i+1]=='r' ){
+      while( i<sz && !fossil_isspace(z[i]) ) i++;
+    }
+  }
+  while( i<sz && fossil_isspace(z[i]) ) i++;
+  blob_append(pOut, &z[i], sz-i);
+}
+
+/*
+** Input z[] is help text for a command zTopic.  Write into pOut all lines of
+** z[] that show the command-line syntax for that command.  Lines written
+** to pOut are lines that begin with out of:
+**
+**     Usage:
+**     or:
+**     > fossil TOPIC
+**
+** Return the number of lines written into pOut.
+*/
+static int simplify_to_usage(
+  const char *z,            /* Full original help text */
+  Blob *pOut,               /* Write simplified help text here */
+  const char *zTopic,       /* The command for which z[] is full help text */
+  int bAbbrevSubcmd         /* z[] uses abbreviated subcommands */
+){
+  ReCompiled *pRe = 0;
+  Blob in, line;
+  int n = 0;
+
+  if( bAbbrevSubcmd ){
+    fossil_re_compile(&pRe, "^(Usage: |   [a-z][-a-z|]+ .*)", 0);
+  }else{
+    fossil_re_compile(&pRe, "^(Usage: | *[Oo]r: +%fossi |>  ?fossil )", 0);
+  }
+  blob_init(&in, z, -1);
+  while( blob_line(&in, &line) ){
+    if( re_match(pRe, (unsigned char*)blob_buffer(&line), blob_strlen(&line)) ){
+      simplify_usage_line(&line, pOut, bAbbrevSubcmd, zTopic);
+      n++;
+    }
+  }
+  re_free(pRe);
+  if( n ) blob_trim(pOut);
+  return n;
+}
+
+/*
+** Input z[] is help text.  Write into pOut all lines of z[] that show
+** command-line options.  Return the number of lines written.
+*/
+static int simplify_to_options(
+  const char *z,            /* Full original help text */
+  Blob *pOut,               /* Write simplified help text here */
+  int bAbbrevSubcmd,        /* z[] uses abbreviated subcommands */
+  const char *zCmd          /* Name of the command that z[] describes */
+){
+  ReCompiled *pRe = 0;
+  Blob txt, line, subsection;
+  int n = 0;
+  int bSubsectionSeen = 0;
+
+  blob_init(&txt, z, -1);
+  blob_init(&subsection, 0, 0);
+  fossil_re_compile(&pRe, "^ +-.*  ", 0);
+  while( blob_line(&txt, &line) ){
+    int len = blob_size(&line);
+    unsigned char *zLine = (unsigned char *)blob_buffer(&line);
+    if( re_match(pRe, zLine, len) ){
+      if( blob_size(&subsection) ){
+        simplify_usage_line(&subsection, pOut, bAbbrevSubcmd, zCmd);
+        blob_reset(&subsection);
+      }
+      blob_appendb(pOut, &line);
+    }else if( len>7 && !fossil_isspace(zLine[0]) && bSubsectionSeen 
+           && sqlite3_strlike("%options:%",blob_str(&line),0)==0 ){
+      subsection = line;
+    }else if( !bAbbrevSubcmd && len>9
+           && (memcmp(zLine,"> fossil ",9)==0 
+               || memcmp(zLine,">  fossil",9)==0) ){
+      subsection = line;
+      bSubsectionSeen = 1;
+    }else if( bAbbrevSubcmd && len>5 && memcmp(zLine,"   ",3)==0
+           && fossil_isalpha(zLine[3]) ){
+      subsection = line;
+      bSubsectionSeen = 1;
+    }else if( len>1 && !fossil_isspace(zLine[0]) && bSubsectionSeen ){
+      blob_reset(&subsection);
+    }
+  }
+  re_free(pRe);
+  blob_trim(pOut);
+  blob_reset(&subsection);
+  return n;
+}
+
+
+
 static void multi_column_list(const char **azWord, int nWord){
   int i, j, len;
   int mxLen = 0;
@@ -1109,14 +1385,12 @@ static void command_list(int cmdMask, int verboseFlag, int useHtml){
 /* @-comment: # */
 static const char zOptions[] =
 @ Command-line options common to all commands:
-@ 
+@
 @   --args FILENAME         Read additional arguments and options from FILENAME
 @   --case-sensitive BOOL   Set case sensitivity for file names
 @   --cgitrace              Active CGI tracing
-@   --chdir PATH            Change to PATH before performing any operations 
-@   --comfmtflags VALUE     Set comment formatting flags to VALUE
-@   --comment-format VALUE  Alias for --comfmtflags
-@   --errorlog FILENAME     Log errors to FILENAME 
+@   --chdir PATH            Change to PATH before performing any operations
+@   --errorlog FILENAME     Log errors to FILENAME
 @   --help                  Show help on the command rather than running it
 @   --httptrace             Trace outbound HTTP requests
 @   --localtime             Display times using the local timezone
@@ -1136,51 +1410,64 @@ static const char zOptions[] =
 /*
 ** COMMAND: help
 **
-** Usage: %fossil help [OPTIONS] [TOPIC]
+** Usage: %fossil help [OPTIONS] [TOPIC] [SUBCOMMAND]
 **
 ** Display information on how to use TOPIC, which may be a command, webpage, or
 ** setting.  Webpage names begin with "/".  If TOPIC is omitted, a list of
-** topics is returned.
+** topics is returned.  If there is an extra argument after TOPIC, it is
+** the name of a subcommand, in which case only the help text for that one
+** subcommand is shown.
 **
 ** The following options can be used when TOPIC is omitted:
 **
 **    -a|--all          List both common and auxiliary commands
+**    -e|--everything   List all help on all topics
+**    -f|--full         List full set of commands (including auxiliary
+**                      and unsupported "test" commands), options,
+**                      settings, and web pages
 **    -o|--options      List command-line options common to all commands
 **    -s|--setting      List setting names
 **    -t|--test         List unsupported "test" commands
 **    -v|--verbose      List both names and help text
 **    -x|--aux          List only auxiliary commands
 **    -w|--www          List all web pages
-**    -f|--full         List full set of commands (including auxiliary
-**                      and unsupported "test" commands), options,
-**                      settings, and web pages
-**    -e|--everything   List all help on all topics
 **
 ** These options can be used when TOPIC is present:
 **
-**    -h|--html         Format output as HTML rather than plain text
 **    -c|--commands     Restrict TOPIC search to commands
+**    -h|--html         Format output as HTML rather than plain text
+**    -o|--options      Show command-line options for TOPIC
+**    --raw             Output raw, unformatted help text
+**    -u|--usage        Show a succinct usage summary, not full help text
+**
+** See also:  [[usage]], [[options]], [[search]] with the -h option
 */
 void help_cmd(void){
   int rc;
-  int mask = CMDFLAG_ANY;
-  int isPage = 0;
-  int verboseFlag = 0;
-  int commandsFlag = 0;
-  const char *z;
-  const char *zCmdOrPage;
-  const CmdOrPage *pCmd = 0;
-  int useHtml = 0;
-  const char *zTopic;
-  Blob txt;
+  int mask = CMDFLAG_ANY;        /* Mask of help topic types */
+  int isPage = 0;                /* True if TOPIC is a page */
+  int verboseFlag = 0;           /* -v option */
+  int commandsFlag = 0;          /* -c option */
+  const char *z;                 /* Original, untranslated help text */
+  const char *zCmdOrPage;        /* "command" or "page" or "setting" */
+  const CmdOrPage *pCmd = 0;     /* ptr to aCommand[] entry for TOPIC */
+  int useHtml = 0;               /* -h option */
+  int bUsage;                    /* --usage */
+  int bRaw;                      /* --raw option */
+  int bOptions;                  /* --options */
+  const char *zTopic;            /* TOPIC argument */
+  const char *zSubtopic = 0;     /* SUBTOPIC argument */
+  Blob subtext1, subtext2, s3;   /* Subsets of z[] containing subtopic/usage */
+  Blob txt;                      /* Text after rendering */
+  int bAbbrevSubcmd = 0;         /* Help text uses abbreviated subcommands */
+
   verboseFlag = find_option("verbose","v",0)!=0;
   commandsFlag = find_option("commands","c",0)!=0;
   useHtml = find_option("html","h",0)!=0;
-  if( find_option("options","o",0) ){
-    fossil_print("%s", zOptions);
-    return;
-  }
-  else if( find_option("all","a",0) ){
+  bRaw = find_option("raw",0,0)!=0;
+  bOptions = find_option("options","o",0)!=0;
+  bUsage = find_option("usage","u",0)!=0;
+  if( find_option("all","a",0) ){
     command_list(CMDFLAG_1ST_TIER | CMDFLAG_2ND_TIER, verboseFlag, useHtml);
     return;
   }
@@ -1212,8 +1499,10 @@ void help_cmd(void){
     command_list(CMDFLAG_WEBPAGE, verboseFlag, useHtml);
     fossil_print("\nfossil test commands (unsupported):\n\n");
     command_list(CMDFLAG_TEST, verboseFlag, useHtml);
-    fossil_print("\n");
-    version_cmd();
+    if ( !verboseFlag ) {
+      fossil_print("\n");
+      version_cmd();
+    }
     return;
   }
   else if( find_option("everything","e",0) ){
@@ -1223,19 +1512,28 @@ void help_cmd(void){
   }
   verify_all_options();
   if( g.argc<3 ){
+    if( bOptions ){
+      fossil_print("%s", zOptions);
+      return;
+    }
     z = g.argv[0];
     fossil_print(
       "Usage: %s help TOPIC\n"
-      "Try \"%s help help\" or \"%s help -a\" for more options\n"
-      "Frequently used commands:\n",
-      z, z, z);
+      "Things to try:\n\n"
+      "   %s help help\n"
+      "   %s help -o\n"
+      "   %s help -a\n"
+      "   %s search -h TOPIC\n\n"
+      "Other common values for TOPIC:\n\n",
+      z, z, z, z, z);
     command_list(CMDFLAG_1ST_TIER,verboseFlag,useHtml);
     if( !verboseFlag ) version_cmd();
     return;
   }
   zTopic = g.argv[2];
+  zSubtopic = g.argc>=4 ? g.argv[3] : 0;
   isPage = ('/' == zTopic[0]) ? 1 : 0;
-  if(isPage){
+  if( isPage ){
     zCmdOrPage = "page";
   }else if( commandsFlag ){
     mask = CMDFLAG_COMMAND;
@@ -1248,6 +1546,10 @@ void help_cmd(void){
     int i, n;
     const char *az[5];
     if( rc==1 ){
+      if( help_is_platform_command(g.argv[2]) ){
+        fossil_print("Not available in this build: %s\n", g.argv[2]);
+        return;
+      }
       fossil_print("unknown %s: %s\n", zCmdOrPage, g.argv[2]);
     }else{
       fossil_print("ambiguous %s prefix: %s\n",
@@ -1258,39 +1560,73 @@ void help_cmd(void){
     for(i=0; i<n; i++){
       fossil_print("  *  %s\n", az[i]);
     }
-    fossil_print("Also consider using:\n");
-    fossil_print("   fossil help TOPIC     ;# show help on TOPIC\n");
-    fossil_print("   fossil help -a        ;# show all commands\n");
-    fossil_print("   fossil help -w        ;# show all web-pages\n");
-    fossil_print("   fossil help -s        ;# show all settings\n");
-    fossil_print("   fossil help -o        ;# show global options\n");
-    fossil_exit(1);
+    fossil_print("Other commands to try:\n");
+    fossil_print("   fossil search -h PATTERN  ;# search all help text\n");
+    fossil_print("   fossil help -a            ;# show all commands\n");
+    fossil_print("   fossil help -w            ;# show all web-pages\n");
+    fossil_print("   fossil help -s            ;# show all settings\n");
+    fossil_print("   fossil help -o            ;# show global options\n");
+    return;
   }
+  bAbbrevSubcmd = (pCmd->eCmdFlags & CMDFLAG_ABBREVSUBCMD)!=0;
   z = pCmd->zHelp;
   if( z==0 ){
     fossil_fatal("no help available for the %s %s",
                  pCmd->zName, zCmdOrPage);
   }
-  if( pCmd->eCmdFlags & CMDFLAG_SETTING ){
-    fossil_print("Setting: \"%s\"%s\n\n",
-         pCmd->zName,
+  blob_init(&subtext1, 0, 0);
+  blob_init(&subtext2, 0, 0);
+  blob_init(&s3, 0, 0);
+  if( zSubtopic!=0 ){
+    if( simplify_to_subtopic(z, &subtext1, zTopic, zSubtopic, bAbbrevSubcmd) ){
+      z = blob_str(&subtext1);
+    }else{
+      fossil_print("No subtopic \"%s\" for \"%s\".\n", zSubtopic, zTopic);
+      bUsage = 1;
+      zSubtopic = 0;
+    }
+  }
+  if( bUsage ){
+    if( simplify_to_usage(z, &subtext2, zTopic, bAbbrevSubcmd) ){
+      z = blob_str(&subtext2);
+    }else{
+      bUsage = 0;
+    }
+  }
+  if( bOptions ){
+    simplify_to_options(z, &s3, bAbbrevSubcmd, zTopic);
+    z = blob_str(&s3);
+  }
+  if( pCmd && pCmd->eCmdFlags & CMDFLAG_SETTING ){
+    const Setting *pSetting = db_find_setting(pCmd->zName, 0);
+    char *zDflt = 0;
+    if( pSetting!=0 && pSetting->def!=0 && *pSetting->def!=0 ){
+      zDflt = mprintf(" (default: %s)", pSetting->def);
+    }
+    fossil_print("Setting: \"%s\"%s%s\n\n",
+         pCmd->zName, zDflt!=0 ? zDflt : "",
          (pCmd->eCmdFlags & CMDFLAG_VERSIONABLE)!=0 ? " (versionable)" : ""
     );
+    fossil_free(zDflt);
   }
   blob_init(&txt, 0, 0);
-  if( useHtml ){
+  if( bRaw ){
+    blob_append(&txt, z, -1);
+  }else if( useHtml ){
     help_to_html(z, &txt);
   }else{
-    help_to_text(z, &txt);
+    help_to_text(z, &txt, bUsage || zSubtopic!=0);
   }
-  fossil_print("%s\n", blob_str(&txt));
+  if( blob_strlen(&txt)>0 ) fossil_print("%s\n", blob_str(&txt));
   blob_reset(&txt);
+  blob_reset(&subtext1);
+  blob_reset(&subtext2);
 }
 
 /*
 ** Return a pointer to the setting information array.
 **
-** This routine provides access to the aSetting2[] array which is created
+** This routine provides access to the aSetting[] array which is created
 ** by the mkindex utility program and included with <page_index.h>.
 */
 const Setting *setting_info(int *pnCount){
@@ -1298,6 +1634,31 @@ const Setting *setting_info(int *pnCount){
   return aSetting;
 }
 
+/*
+** Return a pointer to a specific Setting entry for the setting named
+** in the argument.  Or return NULL if no such setting exists.
+**
+** The pointer returned points into the middle of the global aSetting[]
+** array that is generated by mkindex.  Use setting_info() to fetch the
+** whole array.  Use this routine to fetch a specific entry.
+*/
+const Setting *setting_find(const char *zName){
+  int iFirst = 0;
+  int iLast = ArraySize(aSetting)-1;
+  while( iFirst<=iLast ){
+    int iCur = (iFirst+iLast)/2;
+    int c = strcmp(aSetting[iCur].name, zName);
+    if( c<0 ){
+      iFirst = iCur+1;
+    }else if( c>0 ){
+      iLast = iCur-1;
+    }else{
+      return &aSetting[iCur];
+    }
+  }
+  return 0;
+}
+
 /*****************************************************************************
 ** A virtual table for accessing the information in aCommand[], and
 ** especially the help-text
@@ -1435,7 +1796,7 @@ static int helptextVtabColumn(
     case 4: { /* formatted */
       Blob txt;
       blob_init(&txt, 0, 0);
-      help_to_text(pPage->zHelp, &txt);
+      help_to_text(pPage->zHelp, &txt, 0);
       sqlite3_result_text(ctx, blob_str(&txt), -1, fossil_free);
       break;
     }
@@ -1472,11 +1833,11 @@ static int helptextVtabEof(sqlite3_vtab_cursor *cur){
 /*
 ** This method is called to "rewind" the helptextVtab_cursor object back
 ** to the first row of output.  This method is always called at least
-** once prior to any call to helptextVtabColumn() or helptextVtabRowid() or 
+** once prior to any call to helptextVtabColumn() or helptextVtabRowid() or
 ** helptextVtabEof().
 */
 static int helptextVtabFilter(
-  sqlite3_vtab_cursor *pVtabCursor, 
+  sqlite3_vtab_cursor *pVtabCursor,
   int idxNum, const char *idxStr,
   int argc, sqlite3_value **argv
 ){
@@ -1501,7 +1862,7 @@ static int helptextVtabBestIndex(
 }
 
 /*
-** This following structure defines all the methods for the 
+** This following structure defines all the methods for the
 ** virtual table.
 */
 static sqlite3_module helptextVtabModule = {
@@ -1528,7 +1889,8 @@ static sqlite3_module helptextVtabModule = {
   /* xSavepoint  */ 0,
   /* xRelease    */ 0,
   /* xRollbackTo */ 0,
-  /* xShadowName */ 0
+  /* xShadowName */ 0,
+  /* xIntegrity  */ 0
 };
 
 
